package org.nakedobjects.nof.reflect.java.annotations;

import java.lang.reflect.Method;
import java.util.regex.Pattern;

import org.nakedobjects.applib.annotation.RegEx;
import org.nakedobjects.noa.annotations.Annotation;
import org.nakedobjects.noa.annotations.RegExAnnotation;
import org.nakedobjects.noa.annotations.RegExAnnotation.RegExProcessor;


public class RegExAnnotationFactory extends AbstractAnnotationFactory {

    public RegExAnnotationFactory() {
        super(RegExAnnotation.class);
    }

    /**
     * In readiness for supporting <tt>@Value</tt> in the future.
     */
    public Annotation process(Class clazz) {
        RegEx annotation = (RegEx) clazz.getAnnotation(RegEx.class);
        return create(annotation);
    }

    public Annotation process(final Method method) {
        RegEx annotation = (RegEx) method.getAnnotation(RegEx.class);
        return create(annotation);
    }

    public Annotation processParams(Method method, int paramNum) {

        java.lang.annotation.Annotation[] parameterAnnotations = method.getParameterAnnotations()[paramNum];
        for (int j = 0; j < parameterAnnotations.length; j++) {
            if (parameterAnnotations[j] instanceof RegEx) {
                return create((RegEx) parameterAnnotations[j]);
            }
        }
        return null;
    }

    private RegExAnnotation create(RegEx annotation) {
        if (annotation == null) {
            return null;
        }

        String validationExpression = annotation.validation();
        boolean caseSensitive = annotation.caseSensitive();
        String formatExpression = annotation.format();
        final RegExAnnotation regExAnnotation = new RegExAnnotation(validationExpression, formatExpression, caseSensitive);
        final Pattern pattern = Pattern.compile(regExAnnotation.validation(),
                !regExAnnotation.caseSensitive() ? Pattern.CASE_INSENSITIVE : 0);
        RegExProcessor regExProcessor = new RegExProcessor() {

            public String format(final String text) {
                return pattern.matcher(text).replaceAll(regExAnnotation.format());
            }

            public boolean matches(String text) {
                return pattern.matcher(text).matches();
            }
        };
        regExAnnotation.setRegExProcessor(regExProcessor);
        return regExAnnotation;
    }

}
