package org.nakedobjects.nof.reflect.java.annotations;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;

import org.nakedobjects.applib.annotation.TypeOf;
import org.nakedobjects.noa.annotations.Annotation;
import org.nakedobjects.noa.annotations.TypeOfAnnotation;


public class TypeOfAnnotationFactory extends AbstractAnnotationFactory {

    public TypeOfAnnotationFactory() {
        super(TypeOfAnnotation.class);
    }

    public Annotation process(Method method) {
        TypeOf annotation = (TypeOf) method.getAnnotation(TypeOf.class);

        if (annotation != null) {
            return new TypeOfAnnotation(annotation.value());
        }

        Type type = method.getGenericReturnType();
        if (type instanceof ParameterizedType) {
            ParameterizedType parameterizedType = (ParameterizedType) type;
            Type[] actualTypeArguments = parameterizedType.getActualTypeArguments();
            if (actualTypeArguments.length == 0) {
                return null;
            }
            Object actualTypeArgument = actualTypeArguments[0];
            if (actualTypeArgument instanceof Class) {
                final Class actualType = (Class) actualTypeArgument;
                return new TypeOfAnnotation(actualType, true);
            }
            if (actualTypeArgument instanceof TypeVariable) {
                TypeVariable typeVariable = (TypeVariable)actualTypeArgument;
                // TODO: what to do here?
                return null;
            }
            
        }

        return null;
    }

}
