package org.nakedobjects.nof.reflect.java.fixture;

import java.util.Enumeration;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.nakedobjects.applib.fixtures.AbstractFixture;
import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.NakedCollection;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.noa.spec.NakedObjectSpecification;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.context.Perspective;
import org.nakedobjects.nof.core.persist.TitleCriteria;
import org.nakedobjects.nof.core.service.ServiceUtil;

/**
 * @deprecated use {@link InstancesFixture} and {@link PerspectiveFixture} instead
 */
public abstract class JavaFixture extends AbstractFixture {
    private static final Logger LOG = Logger.getLogger(JavaFixture.class);
    private final Vector newInstances = new Vector();

    public final Object createInstance(Class cls) {
        return createInstance(cls.getName());
    }
    
    public void resetClock() {
        service.resetClock();
    }

    public void setDate(final int year, final int month, final int day) {
        service.setDate(year, month, day);
    }

    public void setTime(final int hour, final int minute) {
        service.setTime(hour, minute);
    }
    
    protected Object getInstance(final Class cls, final String title) {
        return findInstance(cls, title);
    }




    /**
     * Helper method to create an instance of the given type. Provided for exploration programs that need to
     * set up instances.
     */
    public final Object createInstance(final String className) {
        NakedObjectSpecification spec = NakedObjectsContext.getReflector().loadSpecification(className);
        NakedObject object = NakedObjectsContext.getObjectLoader().createTransientInstance(spec);
        LOG.debug("adding " + object);
        newInstances.addElement(object);
        return object.getObject();
    }


    public final Object findInstance(final Class cls, final String title) {
        Enumeration e = newInstances.elements();
        NakedObjectSpecification spec = NakedObjectsContext.getReflector().loadSpecification(cls);
        Vector instances = new Vector();
        while (e.hasMoreElements()) {
            NakedObject element = (NakedObject) e.nextElement();
            if (element.getSpecification() == spec && element.titleString().equals(title)) {
                instances.addElement(element);
            }
        }

        if (instances.size() > 1) {
            throw new NakedObjectRuntimeException("Found more than one instance of " + cls.getName() + " with title " + title
                    + ": " + instances.size());
        } else if (instances.size() == 1) {
            return ((NakedObject) instances.elementAt(0)).getObject();
        }

        
        NakedCollection collection = NakedObjectsContext.getObjectPersistor().findInstances(new TitleCriteria(spec, title, false));
        if (collection.size() > 1) {
            throw new NakedObjectRuntimeException("Found more than one instance of " + cls.getName() + " with title " + title
                    + ": " + collection.size());
        } else if (collection.size() == 1) {
            return collection.firstElement().getObject();
        }
        
        throw new NakedObjectRuntimeException("Failed to find an instance of " + cls.getName() + " with title " + title);
    }

    private void saveNewObjects(final NakedObjectPersistor persistor) {
        for (int i = 0; i < newInstances.size(); i++) {
            NakedObject object = (NakedObject) newInstances.elementAt(i);

            boolean notPersistent = object.getResolveState().isTransient();
            if (notPersistent) {
                LOG.debug("persisting " + object);
                persistor.makePersistent(object);
            }
        }
        persistor.saveChanges();
        newInstances.removeAllElements();
    }
    
    
    
    
    private final Vector services = new Vector();
    private Perspective perspective;
    
    protected void addExplorationPerspective(String title) {
        addPerspective("exploration", title);
    }
    
    protected void addTemplatePerspective(String title) {
        addPerspective("_template", title);
    }

    
    public void addPerspective(String user, String title) {
        perspective = new Perspective();
        perspective.setName(title);
        perspective.setUserName(user);
        services.add(perspective);
    }

    public void addService(String serviceName) {
        if (perspective == null) {
            throw new NakedObjectRuntimeException("No existing perspective");
        }
        perspective.addToServices(findService(serviceName));
    }
    
    private Object findService(String id) {
        Object[] services;
        services = NakedObjectsContext.getObjectLoader().getServices();
        for (int i = 0; i < services.length; i++) {
            if (id.equals(ServiceUtil.id(services[i]))) {
                return services[i];
            }
        }
        throw new NakedObjectRuntimeException("Failed to find service with id of " + id);
    }

    
    

    public void install() {
        NakedObjectPersistor persistor = NakedObjectsContext.getObjectPersistor();
        saveNewObjects(persistor);
        installPerspective(persistor);
    }
    
    private void installPerspective(final NakedObjectPersistor persistor) {
       Enumeration e = services.elements();
       while (e.hasMoreElements()) {
           Object p = e.nextElement();
           NakedObject adapter = NakedObjectsContext.getObjectLoader().createAdapterForTransient(p, false);
           persistor.makePersistent(adapter);
       }
   }


   /*
    protected void postInstallFixtures(final NakedObjectPersistor persistor) {
        persistor.startTransaction();
        try {
            installPerspective(persistor);
            persistor.endTransaction();
        } catch (RuntimeException e) {
            LOG.error("exception during fixture installation; aborting transaction", e);
            try {
                persistor.abortTransaction();
            } catch (Exception e2) {
                LOG.error("failure during abort", e2);
            }
            throw e;
        }
    }
    */

}
// Copyright (c) Naked Objects Group Ltd.
