package org.nakedobjects.nof.reflect.java.fixture;

import java.util.Enumeration;
import java.util.Vector;

import org.nakedobjects.applib.fixtures.AbstractFixture;
import org.nakedobjects.noa.NakedObjectRuntimeException;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.persist.NakedObjectPersistor;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.context.Perspective;
import org.nakedobjects.nof.core.service.ServiceUtil;


public abstract class PerspectiveFixture extends AbstractFixture {
    private final Vector services = new Vector();
    private Perspective perspective;

    protected void addExplorationPerspective(String title) {
        addPerspective("exploration", title);
    }

    protected void addTemplatePerspective(String title) {
        addPerspective("_template", title);
    }

    protected void addPerspective(String user, String title) {
        perspective = new Perspective();
        perspective.setName(title);
        perspective.setUserName(user);
        services.add(perspective);
    }

    protected void addService(String serviceName) {
        if (perspective == null) {
            throw new NakedObjectRuntimeException("No existing perspective");
        }
        perspective.addToServices(findService(serviceName));
    }

    private Object findService(String id) {
        Object[] services;
        services = NakedObjectsContext.getObjectLoader().getServices();
        for (int i = 0; i < services.length; i++) {
            if (id.equals(ServiceUtil.id(services[i]))) {
                return services[i];
            }
        }
        throw new NakedObjectRuntimeException("Failed to find service with id of " + id);
    }

    public final void install() {
        installPerspective();
        NakedObjectPersistor persistor = NakedObjectsContext.getObjectPersistor();
        Enumeration e = services.elements();
        while (e.hasMoreElements()) {
            Object p = e.nextElement();
            NakedObject adapter = NakedObjectsContext.getObjectLoader().createAdapterForTransient(p, false);
            persistor.makePersistent(adapter);
        }
    }

    protected abstract void installPerspective();

    /*
     * protected void postInstallFixtures(final NakedObjectPersistor persistor) {
     * persistor.startTransaction(); try { installPerspective(persistor); persistor.endTransaction(); } catch
     * (RuntimeException e) { LOG.error("exception during fixture installation; aborting transaction", e); try {
     * persistor.abortTransaction(); } catch (Exception e2) { LOG.error("failure during abort", e2); } throw
     * e; } }
     */

}
// Copyright (c) Naked Objects Group Ltd.
