package org.nakedobjects.nof.reflect.java.reflect;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.nakedobjects.applib.value.Color;
import org.nakedobjects.applib.value.DateTime;
import org.nakedobjects.applib.value.Image;
import org.nakedobjects.applib.value.Money;
import org.nakedobjects.applib.value.MultilineString;
import org.nakedobjects.applib.value.Password;
import org.nakedobjects.applib.value.Time;
import org.nakedobjects.applib.value.TimeStamp;


/**
 * Standard way of determining which fields are to be exposed in a Naked Objects system.
 */
public class DefaultClassStrategy implements ClassStrategy {
    private static Class[] invalidFieldTypes = null;
    private static Map valueTypes = new HashMap();
    // TODO these classes are all repeated twice in JavaReflector (see init() and createValueAdapter)
    static {
        Class[] valueTypesArray = new Class[] { boolean.class, char.class, byte.class, short.class, int.class, long.class,
                float.class, double.class,

                String.class, 
                java.sql.Date.class, java.sql.Time.class, java.util.Date.class,
                java.math.BigDecimal.class, java.math.BigInteger.class,

                Boolean.class, Character.class, Short.class, Integer.class, Long.class, Float.class, Double.class,

                java.awt.Image.class,

                org.nakedobjects.applib.value.Date.class, DateTime.class, Time.class, TimeStamp.class, Money.class, Image.class,
                MultilineString.class, Password.class, Color.class };
        for (int i = 0; i < valueTypesArray.length; i++) {
            valueTypes.put(valueTypesArray[i], valueTypesArray[i]);
        }
    }

    public DefaultClassStrategy() {
        super();
    }

    public Class getClass(final Class cls) {
        return cls;
    }

    public Class[] getInvalidFieldTypes() {
        return invalidFieldTypes;
    }

    public Class[] getValueTypes() {
        final Collection values = valueTypes.values();
        return (Class[]) values.toArray(new Class[] {});
    }

    public void addValueType(final Class cls) {
        valueTypes.put(cls, cls);
    }

    public boolean isValueType(final Class cls) {
        return valueTypes.containsKey(cls);
    }

    public boolean isSystemClass(final Class cls) {
        return false;
    }

}
// Copyright (c) Naked Objects Group Ltd.
