package org.nakedobjects.nof.reflect.java.reflect;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.adapter.NakedObject;
import org.nakedobjects.noa.adapter.ResolveException;
import org.nakedobjects.nof.reflect.peer.ObjectTitle;


public class JavaObjectTitle implements ObjectTitle {
    private final static Logger LOG = Logger.getLogger(JavaObjectTitle.class);

    private Method titleMethod;

    public JavaObjectTitle(final Method titleMethod) {
        this.titleMethod = titleMethod;
    }

    public String title(final NakedObject object) {
        try {
            Object title = titleMethod.invoke(object.getObject(), new Object[0]);
            return title == null ? null : title.toString();
        } catch (InvocationTargetException e) {
            Throwable targetException = e.getTargetException();
            if (targetException instanceof ResolveException) {
                throw (ResolveException) targetException;
            }
            LOG.error("exception executing " + titleMethod, targetException);
        } catch (IllegalAccessException ignore) {
            LOG.error("illegal access of " + titleMethod, ignore);
        }

        return "title error...";
    }

}
// Copyright (c) Naked Objects Group Ltd.
