package org.nakedobjects.nof.reflect.java.reflect;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.nakedobjects.noa.reflect.NakedObjectReflector;
import org.nakedobjects.nof.core.system.ReflectorEnhancementInstaller;
import org.nakedobjects.nof.core.system.ReflectorInstaller;
import org.nakedobjects.nof.core.util.NakedObjectConfiguration;
import org.nakedobjects.nof.reflect.peer.ReflectionPeerFactory;
import org.nakedobjects.nof.reflect.peer.ReflectionPeerFactoryInstaller;
import org.nakedobjects.nof.reflect.transaction.TransactionPeerFactoryInstaller;


public class JavaReflectorInstaller implements ReflectorInstaller {
    private static final Logger LOG = Logger.getLogger(JavaReflectorInstaller.class);
    private Vector enhancements = new Vector();

    public NakedObjectReflector createReflector(final NakedObjectConfiguration configuration) {
        JavaReflector reflector = new JavaReflector();
        if (enhancements.size() == 0) {
            LOG.debug("No reflective peers set up");
        }

        final List factories = new ArrayList();
        for (final Iterator iter = enhancements.iterator(); iter.hasNext();) {
            final ReflectionPeerFactoryInstaller installer = (ReflectionPeerFactoryInstaller) iter.next();
            factories.addAll(Arrays.asList(installer.createFactories()));
        }
        factories.addAll(Arrays.asList(new TransactionPeerFactoryInstaller().createFactories()));

        final ReflectionPeerFactory[] factoryArray = new ReflectionPeerFactory[factories.size()];
        int i = 0;
        for (final Iterator iter = factories.iterator(); iter.hasNext();) {
            factoryArray[i++] = (ReflectionPeerFactory) iter.next();
        }

        reflector.setReflectionPeerFactories(factoryArray);
        return reflector;
    }

    public void addEnhancement(ReflectorEnhancementInstaller enhancement) {
        enhancements.add(enhancement);
    }

    public String getName() {
        return "java";
    }

}

// Copyright (c) Naked Objects Group Ltd.
